﻿-- Macro Profiles
local MPS_Version = "1.5"

MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

--Char specific macros
minMacros = 1
charMacros = MAX_ACCOUNT_MACROS + 1
maxMacros = MAX_ACCOUNT_MACROS+MAX_CHARACTER_MACROS

function MPS:OnInitialize()
   MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Default")
end

function MPS:Echo(text)
   DEFAULT_CHAT_FRAME:AddMessage("Profiles: Macros - "..text)
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreSaveMacroProfile(name)
	MPS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:SaveMacroProfile(name)
	-- No macros!
	local numperglobal,numperchar = GetNumMacros()
	if numperchar == 0 and numperglobal == 0 then MPS:Echo("Nothing to save!") return end
	
	-- Duplicate macro name!
	-- Since we are using the name to identify where its actionBar_pos is, WARN user
	local macro_name_table = {}
	for i=minMacros,maxMacros do -- 1..58 ??
		local macro_name, _, _ = GetMacroInfo(i)
		
		if macro_name_table[macro_name] then
			
			if macro_name_table[macro_name] == 1 then -- Dont display WARN more than once..
				MPS:Echo("WARNING: Duplicate macro name: '"..macro_name.."'. Rename it and save again.")
				macro_name_table[macro_name] = 2
			end
		else
			if macro_name then
				macro_name_table[macro_name] = 1
			end
		end
	end
	
	MPdb:SetProfile(name)
	
	-- Solving /mp save tank X 2
	if MPS:ExistsProfile(name) then MPdb:ResetProfile(name) end
	
	MPS:Echo("Saving macros to profile "..name.."!")
	
	
	-- Cache icons path
	local icons_table = {}
	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
    
	-- Save GLOBAL macros
	for i=minMacros,maxMacros do
		local macro_name, texture, body = GetMacroInfo(i)
		local actionBar_pos = {};
		
		--MPS:Echo(i)
		-- No more macros saved
		if not macro_name then break end
			
		-- Get icon ID instead of path (needed for CreateMacro() in Load!
		texture = string.lower(texture)
        if icons_table[texture] then 
             texture = icons_table[texture]
        end
		
		-- Icon not found, give question mark!
		if (type(texture) ~= "number") then 
			texture = '1'  
		end

		-- Save actionbar position
		for actionID=1, 120 do
			local _, id = GetActionInfo(actionID)
			local abmacro_name, _, _ = GetMacroInfo(id)
			
			if abmacro_name == macro_name then
				table.insert(actionBar_pos , actionID)
			end
		end
		
		-- Save the GLOBAL macro to db
		MPdb.profile[i] = { macro_name,texture,body,actionBar_pos,"1"}
	end
	
	-- Save CHAR macros
	for i=charMacros,maxMacros do
		local macro_name, texture, body = GetMacroInfo(i)
		local actionBar_pos = {};
		
		--MPS:Echo(i)
		-- No more macros saved
		if not macro_name then break end
			
		-- Get icon ID instead of path (needed for CreateMacro() in Load!
		texture = string.lower(texture)
        if icons_table[texture] then 
             texture = icons_table[texture]
        end
		
		-- Icon not found, give question mark!
		if (type(texture) ~= "number") then 
			texture = '1'  
		end

		-- Save actionbar position
		for actionID=1, 120 do
			local _, id = GetActionInfo(actionID)
			local abmacro_name, _, _ = GetMacroInfo(id)
			
			if abmacro_name == macro_name then
				table.insert(actionBar_pos , actionID)
			end
		end
		
		-- Save the CHAR macro to db
		MPdb.profile[i] = { macro_name,texture,body,actionBar_pos,"0"}
	end
		
	MPS:SendMessage("MPS_Profiles_save",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function MPS:Save(...)
	MPS:SaveMacroProfile(...)
end


-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreLoadMacroProfile(name)
	MPS:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:LoadMacroProfile(name)

	-- No such profile saved
	if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return end
	
	-- In combat!
	if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return end

   MPdb:SetProfile(name)
  
   MPS:Echo("Loading macros from profile "..name.."!")
   
   --Get rid of all the current macros
   for i=minMacros,maxMacros do
      DeleteMacro(minMacros) -- DeleteMacro changes index for all other macros
   end 
   
   --Load saved macros profile
   for index, keys in pairs(MPdb.profile) do   
       if keys[1] then    
            if (type(keys[2]) ~= "number") then keys[2] = '1' end -- Icon not found, give question mark!
            
			
			if index >= charMacros then -- CreateMacro must know if its a global or char macro
				CreateMacro(keys[1], keys[2], keys[3], 1);
			else
				CreateMacro(keys[1], keys[2], keys[3], nil);
			end
			
			-- Action bar handling
			for _,v in pairs(keys[4]) do
				ClearCursor()
				
				-- Clear current spot
				PickupAction(v)
				ClearCursor()
				
				-- Restore this spot
				PickupMacro(keys[1])
				PlaceAction(v)
				ClearCursor()
			end
			
       end
   end
   
	if (IsAddOnLoaded("Blizzard_MacroUI")) then
		MacroFrame_Update() -- Blizzards own code
	end
   
	MPS:SendMessage("MPS_Profiles_load",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function MPS:Load(...)
	MPS:LoadMacroProfile(...)
end

function MPS:ExistsProfile(name)
   for i, k in ipairs(MPdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end

function MPS:ListProfiles()
    MPS:Echo(" Available Profiles: ");
    
	for i,k in ipairs(MPdb:GetProfiles()) do
		if k == MPdb:GetCurrentProfile() then
			MPS:Echo(' ' .. k.." (current)")
		else
			MPS:Echo(' ' .. k)
		end
	end
end

function MPS:DeleteProfil(name)

	if (MPS:ExistsProfile(name)==nil)then
		MPS:Echo(name.." doesnt exist.")
		MPS:ListProfiles()
	else
		if name == MPdb:GetCurrentProfile() then
			MPdb:SetProfile('Default')
		end
		MPdb:DeleteProfile(name)
		MPS:Echo(name.." deleted.")
	end
end

--Regexp routine
-- nocase("Hi there!")) >>> 
-- [hH][iI] [tT][hH][eE][rR][eE]!
function MPS:nocase (s)
   s = string.gsub(s, "%a", function (c)
         return string.format("[%s%s]", string.lower(c),
            string.upper(c))
   end)
   return s
end